package Game.Entity
{
	import flash.geom.Point;
	
	import Game.GameScene;
	
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;

	public class Castle extends GameEntity
	{
		public var x:Number;
		public var y:Number;
		public var team:int;
		public var image:Image;
		
		public var region:Vector.<Point>;
		public var regionSprite:Sprite;
		public var regionQuads:Vector.<Quad>;
		public var regionHighlight:Sprite;
		public var adjacent:Vector.<Castle>;
		
		public var hp:Number;
		public var spawnTimer:Number;
		public var arrowTimer:Number;
		public var diminish:Number;
		
		public var units:Vector.<Unit>;
		public var unitIndex:int;
		
		public var hpBg:Quad;
		public var hpBar:Quad;
		public var hpSprite:Sprite;
		public var apBg:Quad;
		public var apBar:Quad;
		public var apSprite:Sprite;
		
		public var aiTimer:Number;
		
		public function Castle()
		{
			image = new Image(Root.assets.getTexture("village"));
			image.pivotX = image.width*0.5;
			image.pivotY = image.height*0.75;
			
			hpSprite = new Sprite();
			hpBg = new Quad(50, 6, 0x000000);
			hpBar = new Quad(48, 4, 0xffffff);
			hpBar.x = hpBar.y = 1;
			hpSprite.addChild(hpBg);
			hpSprite.addChild(hpBar);
			
			units = new Vector.<Unit>();
			unitIndex = 0;
		}
		
		public function initCastle(game:GameScene, x:Number, y:Number, team:int):void {
			super.init(game);
			this.x = x;
			this.y = y;
			this.team = team;
			spawnTimer = 0;
			arrowTimer = 0;
			hp = 0;
			diminish = 1.0;
			
			image.x = x;
			image.y = y;
			image.color = Globals.colors[team];
			game.unitLayer.addChild(image);
			
			hpSprite.x = x-25;
			hpSprite.y = y-60;
			hpSprite.visible = false;
			game.barLayer.addChild(hpSprite);
			aiTimer = 1.0 + Math.random();
		}
		
		public function initRegion(region:Vector.<Point>):void {
			this.region = region;
			regionSprite = new Sprite();
			regionQuads = new Vector.<Quad>();
			var l:int = region.length-1;
			for(var i:int = 0; i < l; i++) {
				var quad:Quad = new Quad(10, 10, Globals.borders[team]);
				quad.setVertexPosition(0, region[i].x, region[i].y);
				quad.setVertexPosition(1, region[i+1].x, region[i+1].y);
				quad.setVertexPosition(2, x, y);
				quad.setVertexPosition(3, x, y);
				regionQuads.push(quad);
				regionSprite.addChild(quad);
			}
			
			quad = new Quad(10, 10, Globals.borders[team]);
			quad.setVertexPosition(0, region[i].x, region[i].y);
			quad.setVertexPosition(1, region[0].x, region[0].y);
			quad.setVertexPosition(2, x, y);
			quad.setVertexPosition(3, x, y);
			regionQuads.push(quad);
			regionSprite.addChild(quad);
			
			
			//highlight
			regionHighlight = new Sprite();
			l = region.length-1;
			for(i = 0; i < l; i++) {
				quad = new Quad(10, 10, 0xffffff);
				quad.setVertexPosition(0, region[i].x, region[i].y);
				quad.setVertexPosition(1, region[i+1].x, region[i+1].y);
				quad.setVertexPosition(2, x, y);
				quad.setVertexPosition(3, x, y);
				regionHighlight.addChild(quad);
			}
			quad = new Quad(10, 10, 0xffffff);
			quad.setVertexPosition(0, region[i].x, region[i].y);
			quad.setVertexPosition(1, region[0].x, region[0].y);
			quad.setVertexPosition(2, x, y);
			quad.setVertexPosition(3, x, y);
			regionHighlight.addChild(quad);
			
			regionSprite.visible = false;
			regionHighlight.visible = false;
			game.regions.addChild(regionSprite);
			game.highlights.addChild(regionHighlight);
		}
		
		public function initAdjacent(points:Vector.<Point>):void {
			adjacent = new Vector.<Castle>();
			var l:int = game.castles.activeIndex;
			for(var i:int = 0; i < l; i++) {
				if(game.castles.active[i] == this) continue;
				var k:int = points.length;
				for(var j:int = 0; j < k; j++) {
					if(game.castles.active[i].x == points[j].x && game.castles.active[i].y == points[j].y) 
						if(game.getDistance(x, y, points[j].x, points[j].y) < 500) adjacent.push(game.castles.active[i]);
				}
			}
		}
		
		public override function deInit():void {
			game.unitLayer.removeChild(image);
			game.regions.removeChild(regionSprite);
			game.highlights.removeChild(regionHighlight);
		}
		
		public override function update(dt:Number):void {
			if(team > 0 && Globals.teamPops[team] < Globals.teamCaps[team]) {
				spawnTimer += dt*0.25;
				if(spawnTimer >= 1.0) {
					spawnTimer = 0;
					makeUnit();
				}
			}
			
			if(team > 0) {
				arrowTimer -= dt;
				if(arrowTimer <= 0) {
					var target:Unit = getClosestEnemy();
					if(target) {
						arrowTimer = 0.5;
						game.addArrow(x+Math.random()*40-20, y-Math.random()*20, target);
						GS.playArrow();
					}
				}
			}
			
			hpSprite.visible = false;
			if(hp > 0 && hp < 100) hpSprite.visible = true;
			if(team == 0) hpBar.color = 0xffffff;
			else hpBar.color = Globals.colors[team];
			hpBar.scaleX = hp/100;
			
			//ai
			if(team > 1) {
				aiTimer -= dt;
				if(aiTimer <= 0) {
					aiTimer = 1.0 + Math.random()*2;
					var moved:Boolean = false;
					var l:int = adjacent.length;
					for(var i:int = 0; i < l; i++) {
						if(adjacent[i].team != team) {
							if(adjacent[i].team > 0 && countUnits() < adjacent[i].countUnits()) continue;
							else {
								moveUnits(adjacent[i]);
								moved = true;
							}
						}
					}
					if(!moved) {
						l = adjacent.length;
						for(i = 0; i < l; i++) {
							if(adjacent[i].team == team) {
								moveUnits(adjacent[i]);
							}
						}
					}
				}
			}
		}
		
		public function makeUnit():void {
			var unit:Unit = game.addUnit(x, y, team, this);
			unit.move(x + Math.random()*200-100, y + Math.random()*200-100);
			var overflow:int = 0;
			while((game.getDistance(x, y, unit.tx, unit.ty) < 40 || unit.tx < 20 || unit.tx > Globals.sw-20 || unit.ty < 20 || unit.ty > Globals.sh-20 || !contains(unit.tx, unit.ty)) && overflow < 100) {
				unit.move(x + Math.random()*200-100, y + Math.random()*200-100);
				overflow++;
			}
		}
		
		public function setTeam(team:int):void {
			this.team = team;
			if(team > 0) image.texture = Root.assets.getTexture("castle");
			else image.texture = Root.assets.getTexture("village");
			image.readjustSize();
			image.width = image.texture.width;
			image.height = image.texture.height;
			image.pivotX = image.width*0.5;
			image.pivotY = image.height*0.75;
			image.color = Globals.colors[team];
			
			var l:int = regionQuads.length;
			for(var i:int = 0; i < l; i++) {
				regionQuads[i].color = Globals.borders[team];
			}
			if(team > 0) regionSprite.visible = true;
			else regionSprite.visible = false;
		}
		
		public function moveUnits(castle:Castle):void {
			units.length = unitIndex = 0;
			var l:int = game.units.activeIndex;
			for(var i:int = 0; i < l; i++) {
				var unit:Unit = game.units.active[i] as Unit;
				if(unit.castle == this && unit.team == team && contains(unit.x, unit.y)) {
					units[unitIndex++] = unit;
				}
			}
			
			l = unitIndex;
			for(i = 0; i < l*0.5; i++) {
				unit = units[i];
				unit.moveTo(castle);
			}
		}
		
		public function countUnits():Number {
			var count:int = 0;
			var l:int = game.units.activeIndex;
			for(var i:int = 0; i < l; i++) {
				var unit:Unit = game.units.active[i] as Unit;
				if(unit.castle == this && unit.team == team && contains(unit.x, unit.y)) {
					count++;
				}
			}
			return count;
		}
		
		public function scatter(team:int):void {
			units.length = unitIndex = 0;
			var l:int = game.units.activeIndex;
			for(var i:int = 0; i < l; i++) {
				var unit:Unit = game.units.active[i] as Unit;
				if(unit.castle == this && unit.team == team) {
					units[unitIndex++] = unit;
				}
			}
			
			l = unitIndex;
			for(i = 0; i < l; i++) {
				unit = units[i];
				unit.moveTo(this);
			}
		}
		
		public function getClosestEnemy():Unit {
			var min:Number = 999999999;
			var closest:Unit;
			
			var l:int = game.units.activeIndex
			for(var i:int = 0; i < l; i++) {
				var unit:Unit = game.units.active[i] as Unit;
				if(unit.team == team) continue;
				if(unit.hp <= 0) continue;
				if(unit.castle != this) continue;
				if(!contains(unit.x, unit.y)) continue;
				var dx:Number = x - unit.x;
				var dy:Number = y - unit.y;
				var dist:Number = dx*dx + dy*dy;
				if(dist < min) {
					min = dist;
					closest = unit;
				}
			}
			
			return closest;
		}
		
		public function contains(x:Number, y:Number):Boolean {
			var inside:Boolean = false;
			var l:int = region.length;
			for(var i:int = 0, j:int = l-1; i < l; j = i++) {
				if ( ((region[i].y > y) != (region[j].y > y)) && (x < (region[j].x - region[i].x) * (y-region[i].y) / (region[j].y - region[i].y) + region[i].x) ) inside = !inside;
			}
			return inside;
		}
	}
}