package Game.Entity
{
	import Game.GameScene;
	
	import starling.animation.Transitions;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;

	public class Flag extends GameEntity
	{
		public var x:Number;
		public var y:Number;
		public var dx:Number;
		public var dy:Number;
		public var team:int;
		public var image:Image;
		public var flagImage:Image;
		public var sprite:Sprite;
		public var moveTimer:Number;
		
		public var waypoints:Vector.<Waypoint>;
		public var waypointPool:Vector.<Waypoint>;
		public var waypointIndex:int;
		public var waypointPoolIndex:int;
		
		public var enemyFlag:Flag;
		public var enemyCastle:Castle;
		public var castle:Castle;
		public var state:int;
		
		public static const STATE_IDLE:int = 0;
		public static const STATE_BUILD:int = 1;
		public static const STATE_ATTACK:int = 2;
		
		public function Flag()
		{
			x = y = dx = dy = 0;
			image = new Image(Root.assets.getTexture("unit02"));
			image.pivotX = 48;
			image.pivotY = 48;
			
			flagImage = new Image(Root.assets.getTexture("flag"));
			flagImage.pivotX = 63;
			flagImage.pivotY = 64;
			flagImage.x = 3;
			flagImage.y = -28;
			
			sprite = new Sprite();
			sprite.addChild(image);
			sprite.addChild(flagImage);
			
			waypoints = new Vector.<Waypoint>();
			waypointPool = new Vector.<Waypoint>();
			waypointIndex = 0;
			waypointPoolIndex = 0;
		}
		
		public function initFlag(game:GameScene, x:Number, y:Number, team:int):void {
			super.init(game);
			this.x = x;
			this.y = y;
			this.team = team;
			moveTimer = 0;
			state = STATE_IDLE;
			
			sprite.x = x;
			sprite.y = y;
			image.color = Globals.colors[team];
			flagImage.color = Globals.colors[team];
			game.unitLayer.addChild(sprite);
		}
		
		public override function deInit():void {
			game.unitLayer.removeChild(sprite);
		}
		
		public override function update(dt:Number):void {
			this.x += dx*dt;
			this.y += dy*dt;
			dx -= dx*dt*10;
			dy -= dy*dt*10;
			
			getCurrentCastle();
			if(castle.team == team) state == STATE_IDLE;
			else if(castle.team == 0) state == STATE_BUILD;
			else if(castle.team != team) state == STATE_ATTACK;
			
			var moving:Boolean = false;
			if(waypointIndex > 0) {
				var waypoint:Waypoint = waypoints[0];
				var dist:Number = game.getDistance(this.x, this.y, waypoint.x, waypoint.y);
				if(dist > 5) {
					var angle:Number = game.getAngle(this.x, this.y, waypoint.x, waypoint.y);
					dx += Math.cos(angle)*dt*300;
					dy += Math.sin(angle)*dt*300;
				} else {
					shiftWaypoints();
				}
				moving = true;
			}
			
			if(moving) {
				if(moveTimer == 0) {
					Starling.juggler.tween(image, 0.05, {scaleY:0.9, scaleX:1.1, transition:Transitions.EASE_OUT});
					Starling.juggler.tween(image, 0.225, {scaleY:1.1, scaleX:0.9, y:-10, transition:Transitions.EASE_OUT, delay:0.05});
					Starling.juggler.tween(image, 0.224, {scaleY:1.0, scaleX:1.0, y:0, transition:Transitions.EASE_IN, delay:0.275});
					Starling.juggler.tween(flagImage, 0.1, {scaleY:0.9, scaleX:1.1, y:-26,  transition:Transitions.EASE_OUT});
					Starling.juggler.tween(flagImage, 0.225, {scaleY:1.1, scaleX:0.9, y:-38, transition:Transitions.EASE_OUT, delay:0.1});
					Starling.juggler.tween(flagImage, 0.219, {scaleY:1.0, scaleX:1.0, y:-28, transition:Transitions.EASE_IN, delay:0.28});
				}
				moveTimer += dt*2;
				if(moveTimer > 1.0) moveTimer = 0;
				if(dx > 0) sprite.scaleX = 1;
				else sprite.scaleX = -1;
			}
			else if(Math.abs(dx)+Math.abs(dy) < 0.01) moveTimer = 0;
			
			sprite.x = this.x;
			sprite.y = this.y;
		}
		
		public function addWaypoint(x:Number, y:Number):void {
			var waypoint:Waypoint;
			if(waypointPoolIndex > 0) waypoint = waypointPool[--waypointPoolIndex];
			else waypoint = new Waypoint();
			waypoint.init(x, y, 0);
			waypoints[waypointIndex++] = waypoint;
		}
		
		public function getDistFromLastWaypoint(x:Number, y:Number):Number {
			if(waypointIndex == 0) return game.getDistance(this.x, this.y, x, y);
			else return game.getDistance(waypoints[waypointIndex-1].x, waypoints[waypointIndex-1].y, x, y);
		}
		
		public function clearWaypoints():void {
			while(waypointIndex > 0) {
				waypointPool[waypointPoolIndex++] = waypoints[--waypointIndex];
			}
		}
		
		public function shiftWaypoints():void {
			var l:int = waypointIndex-1;
			for(var i:int = 0; i < l; i++) {
				waypoints[i] = waypoints[i+1];
			}
			waypointIndex--;
		}
		
		public function getCurrentCastle():void {
			var l:int = game.castles.activeIndex;
			for(var i:int = 0; i < l; i++) {
				if(game.castles.active[i].contains(x, y)) {
					castle = game.castles.active[i] as Castle;
					return;
				}
			}
		}
	}
}