package Game.Entity
{
	import Game.GameScene;
	
	import starling.animation.Transitions;
	import starling.core.Starling;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.Sprite;

	public class Unit extends GameEntity
	{
		public var x:Number;
		public var y:Number;
		public var dx:Number;
		public var dy:Number;
		public var tx:Number;
		public var ty:Number;
		public var team:int;
		public var frame:int;
		public var image:Image;
		public var sprite:Sprite;
		public var castle:Castle;
		
		public var hp:Number;
		
		public var moveTimer:Number;
		public var frameTimer:Number;
		public var textures:Array;
		
		public var state:int;
		public static const STATE_IDLE:int = 0;
		public static const STATE_MOVE:int = 1;
		public static const STATE_BUILD:int = 2;
		public static const STATE_ATTACK:int = 3;
		public static const STATE_HURT:int = 4;
		public static const STATE_DEATH:int = 5;
		
		public function Unit()
		{
			x = y = dx = dy = tx = ty = 0;
			textures = [Root.assets.getTexture("unit01"), Root.assets.getTexture("unit02"), Root.assets.getTexture("unit03"), Root.assets.getTexture("unit04"),
				Root.assets.getTexture("unit05"),Root.assets.getTexture("unit06"), Root.assets.getTexture("unit07"), Root.assets.getTexture("unit08"),
				Root.assets.getTexture("unit09"),Root.assets.getTexture("unit10"), Root.assets.getTexture("unit11")
			];
			image = new Image(textures[2]);
			image.pivotX = 48;
			image.pivotY = 48;
			sprite = new Sprite();
			sprite.addChild(image);
		}
		
		public function initUnit(game:GameScene, x:Number, y:Number, team:int, castle:Castle):void {
			super.init(game);
			this.x = tx = x;
			this.y = ty = y;
			this.team = team;
			this.castle = castle;
			state = STATE_IDLE;
			moveTimer = 0;
			frameTimer = 0;
			frame = 0;
			hp = 100;
			
			image.x = image.y = 0;
			sprite.x = x;
			sprite.y = y;
			image.color = Globals.colors[team];
			game.unitLayer.addChild(sprite);
		}
		
		public override function deInit():void {
			game.unitLayer.removeChild(sprite);
		}
		
		public override function update(dt:Number):void {
			this.x += dx*dt;
			this.y += dy*dt;
			dx -= dx*dt*10;
			dy -= dy*dt*10;
			sprite.x = x;
			sprite.y = y;
			
			if(hp <= 0) {
				frameTimer += dt*3;
				if(frameTimer >= 1.0) {
					image.texture = textures[7];
					image.x = x;
					image.y = y;
					game.deadLayer.addImage(image);
					image.x = image.y = 0;
					active = false;
				}
				return;
			}
			
			if(target && (target.hp <= 0 || !target.active)) {
				target = null;
				moveTo(castle);
			}
			
			//collisionCheck();
			if(castle.contains(x, y)) {
				target = getClosestEnemy();
				var building:Boolean = false;
				if(castle.team == 0 && target && game.fastDist(x, y, target.x, target.y) > 10000) target = null;
				if(target) {
					tx = target.x;
					ty = target.y;
					if(game.fastDist(x, y, tx, ty) < 900) {
						if(cooldown > 0) {
							cooldown -= dt;
							return;
						}
						frameTimer += dt*6;
						if(frameTimer >= 1.0) {
							frameTimer = 0;
							frame++;
							if(frame == 4) {
								target.hit(40);
								angle = game.getAngle(x, y, tx, ty);
								target.dx += Math.cos(angle)*20;
								target.dy += Math.sin(angle)*20;
								if(target.hp <= 0) {
									moveTo(castle);
									target = null;
								}
								GS.playSlash();
							}
							if(frame > 5) {
								frame = 2;
								cooldown = 1.0;
							}
							image.texture = textures[frame];
						}
						return;
					}
				} else {
					if(castle.team == 0 || (castle.team == team && castle.hp < 100)) {
						if(game.fastDist(x, y, tx, ty) < 400) {
							if(image.texture == textures[2]) {
								frame = 8;
								image.texture = textures[8];
							}
							frameTimer += dt*6;
							if(frameTimer >= 1.0) {
								frameTimer = 0;
								frame++;
								if(frame == 9) GS.playHit();
								if(frame > 10) frame = 8;
								image.texture = textures[frame];
							}
							castle.hp += dt;
							if(castle.hp >= 100) {
								castle.hp = 100;
								if(castle.team != team) {
									castle.setTeam(team);
									GS.playBuild();
								}
							}
						}
					}
					else if(castle.team != team) {
						tx = castle.x;
						ty = castle.y;
						if(game.fastDist(x, y, tx, ty) < 2500) {
							if(cooldown > 0) {
								cooldown -= dt;
								return;
							}
							frameTimer += dt*6;
							if(frameTimer >= 1.0) {
								frameTimer = 0;
								frame++;
								if(frame == 4) {
									castle.hp -= 1;
									if(castle.hp <= 0) {
										castle.hp = 0;
										castle.setTeam(0);
										castle.scatter(team);
										GS.playExplode();
										//moveTo(castle);
									} else GS.playSlash();
								}
								if(frame > 5) {
									frame = 2;
									cooldown = 1.0;
								}
								image.texture = textures[frame];
							}
							return;
						}
					}
					else if(image.texture != textures[2]) image.texture = textures[2];
				}
			} else if(!castle.contains(tx, ty)) {
				tx = castle.x + Math.random()*200-100;
				ty = castle.y + Math.random()*200-100;
				var overflow:int = 0;
				while((game.getDistance(x, y, tx, ty) < 40 || tx < 20 || tx > Globals.sw-20 || ty < 20 || ty > Globals.sh-20 || !castle.contains(tx, ty)) && overflow < 100) {
					tx = castle.x + Math.random()*200-100;
					ty = castle.y + Math.random()*200-100;
					overflow++;
				}
			}
			
			if(game.fastDist(x, y, tx, ty) > 25) {
				if(image.texture != textures[2]) image.texture = textures[2];
				var angle:Number =  game.getAngle(x, y, tx, ty);
				dx += Math.cos(angle)*500*dt;
				dy += Math.sin(angle)*500*dt;
				
				if(moveTimer == 0) {
					Starling.juggler.tween(image, 0.05, {scaleY:0.9, scaleX:1.1, transition:Transitions.EASE_OUT});
					Starling.juggler.tween(image, 0.225, {scaleY:1.1, scaleX:0.9, y:-10, transition:Transitions.EASE_OUT, delay:0.05});
					Starling.juggler.tween(image, 0.224, {scaleY:1.0, scaleX:1.0, y:0, transition:Transitions.EASE_IN, delay:0.275});
					GS.playJump();
				}
				moveTimer += dt*2;
				if(moveTimer > 1.0) moveTimer = 0;
				if(dx > 0) sprite.scaleX = 1;
				else sprite.scaleX = -1;
			}
			else if(Math.abs(dx)+Math.abs(dy) < 0.01) moveTimer = 0;
		}
		
		public var target:Unit;
		public var cooldown:Number = 0;
		
		public function move(x:Number, y:Number):void {
			this.tx = x;
			this.ty = y;
			state = STATE_MOVE;
		}
		
		public function moveTo(castle:Castle):void {
			this.castle = castle;
			tx = castle.x + Math.random()*200-100;
			ty = castle.y + Math.random()*200-100;
			var overflow:int = 0;
			while((game.getDistance(x, y, tx, ty) < 40 || tx < 20 || tx > Globals.sw-20 || ty < 20 || ty > Globals.sh-20 || !castle.contains(tx, ty)) && overflow < 100) {
				tx = castle.x + Math.random()*200-100;
				ty = castle.y + Math.random()*200-100;
				overflow++;
			}
		}
		
		public function hit(dmg:Number):void {
			hp -= dmg;
			if(hp <= 0) {
				hp = 0;
				frameTimer = 0;
				image.texture = textures[6];
			}
		}
		
		public function collisionCheck():void {
			var l:int = game.units.activeIndex
			for(var i:int = 0; i < l; i++) {
				var unit:Unit = game.units.active[i] as Unit;
				if(unit == this) continue;
				if(unit.hp <= 0) continue;
				var dx:Number = x - unit.x;
				var dy:Number = y - unit.y;
				if(dx > 20 || dx < -20 || dy > 20 || dy < -20) continue;
				var dist:Number = Math.sqrt(dx*dx + dy*dy);
				if(dist < 20) {
					var angle:Number = Math.atan2(dy, dx);
					var cos_x:Number = Math.cos(angle)*dist*0.5;
					var sin_y:Number = Math.sin(angle)*dist*0.5;
					dx += cos_x;
					dy += sin_y;
					unit.dx -= cos_x;
					unit.dy -= sin_y;
				}
				
			}
		}
		
		public function getClosestEnemy():Unit {
			var min:Number = 999999999;
			var closest:Unit;
			
			var l:int = game.units.activeIndex
			for(var i:int = 0; i < l; i++) {
				var unit:Unit = game.units.active[i] as Unit;
				if(unit == this) continue;
				if(unit.team == team) continue;
				if(unit.hp <= 0) continue;
				if(unit.castle != castle) continue;
				if(!castle.contains(unit.x, unit.y)) continue;
				var dx:Number = x - unit.x;
				var dy:Number = y - unit.y;
				var dist:Number = dx*dx + dy*dy;
				if(dist < min) {
					min = dist;
					closest = unit;
				}
			}
			
			return closest;
		}
	}
}