package Game
{
	import com.nodename.Delaunay.Edge;
	import com.nodename.Delaunay.Voronoi;
	import com.nodename.geom.LineSegment;
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	
	import Game.Entity.Arrow;
	import Game.Entity.Castle;
	import Game.Entity.EntityPool;
	import Game.Entity.Flag;
	import Game.Entity.Unit;
	
	import starling.core.Starling;
	import starling.display.DisplayObject;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.display.QuadBatch;
	import starling.display.Sprite;
	import starling.events.EnterFrameEvent;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	
	public class GameScene extends Sprite
	{	
		public var units:EntityPool;
		public var flags:EntityPool;
		public var arrows:EntityPool;
		public var castles:EntityPool;
		public var entities:Vector.<EntityPool>;
		
		public var regions:Sprite;
		public var gameLayer:Sprite;
		public var unitLayer:Sprite;
		public var arrowLayer:Sprite;
		public var barLayer:Sprite;
		public var highlights:Sprite;
		public var touchQuad:Quad;
		public var deadLayer:QuadBatch
		public var borders:QuadBatch;
		public var paths:QuadBatch;
		public var voronoi:Voronoi;
		
		public var ui_arrow:Image;
		public var victory:Image;
		public var defeat:Image;
		public var gameOver:Boolean;
		
		public function GameScene()
		{
			units = new EntityPool();
			flags = new EntityPool();
			arrows = new EntityPool();
			castles = new EntityPool();
			entities = new Vector.<EntityPool>();
			entities.push(units, flags, arrows, castles);
			
			gameLayer = new Sprite();
			gameLayer.touchable = false;
			//gameLayer.blendMode = "multiply";
			addChild(gameLayer);
			
			var bg:Image = new Image(Root.assets.getTexture("bg"));
			bg.blendMode = "none";
			gameLayer.addChild(bg);
			
			regions = new Sprite();
			regions.blendMode = "multiply";
			regions.alpha = 0.5;
			gameLayer.addChild(regions);
			
			borders = new QuadBatch();
			gameLayer.addChild(borders);
			
			deadLayer = new QuadBatch();
			//deadLayer.alpha = 0.5;
			deadLayer.blendMode = "multiply";
			gameLayer.addChild(deadLayer);
			
			unitLayer = new Sprite();
			gameLayer.addChild(unitLayer);
			
			arrowLayer = new Sprite();
			arrowLayer.alpha = 0.9;
			gameLayer.addChild(arrowLayer);
			
			barLayer = new Sprite();
			gameLayer.addChild(barLayer);
			
			highlights = new Sprite();
			highlights.blendMode = "add";
			highlights.alpha = 0.1;
			gameLayer.addChild(highlights);
			
			ui_arrow = new Image(Root.assets.getTexture("ui_arrow"));
			ui_arrow.color = 0xffffff;
			ui_arrow.pivotY = ui_arrow.height*0.5;
			gameLayer.addChild(ui_arrow);
			ui_arrow.visible = false;
			
			paths = new QuadBatch();
			gameLayer.addChild(paths);
			
			victory = new Image(Root.assets.getTexture("victory"));
			victory.pivotX = victory.width*0.5;
			victory.pivotY = victory.height*0.5;
			victory.x = Globals.sw*0.5;
			victory.y = Globals.sh*0.5;
			victory.visible = false;
			gameLayer.addChild(victory);
			
			defeat = new Image(Root.assets.getTexture("defeat"));
			defeat.pivotX = defeat.width*0.5;
			defeat.pivotY = defeat.height*0.5;
			defeat.x = Globals.sw*0.5;
			defeat.y = Globals.sh*0.5;
			defeat.visible = false;
			gameLayer.addChild(defeat);
			
			var fg:Image = new Image(Root.assets.getTexture("fg"));
			gameLayer.addChild(fg);
			
			touchQuad = new Quad(Globals.sw+20, Globals.sh+20, 0xff0000);
			touchQuad.x = touchQuad.y = -10;
			touchQuad.alpha = 0;
			addChild(touchQuad);
			
			initCastles(12);
			//addFlag(100, 100, 1);
			
			var points:Vector.<Point> = new Vector.<Point>();
			var l:int = castles.active.length;
			for(var i:int = 0; i < l; i++) {
				points.push(new Point(castles.active[i].x, castles.active[i].y));
			}
			var rect:Rectangle = new Rectangle(0, 0, Globals.sw, Globals.sh);
			voronoi = new Voronoi(points, null, rect);
			
			l = castles.active.length;
			for(i = 0; i < l; i++) {
				castles.active[i].initRegion(voronoi.region(points[i]));
				castles.active[i].initAdjacent(voronoi.neighborSitesForSite(points[i]));
				//castles.active[i].initAdjacent(voronoi.delaunayLinesForSite(points[i]));
			}
			
			castles.active[0].setTeam(1);
			castles.active[0].hp = 100;
			for(i = 0; i < 5; i++) castles.active[0].makeUnit();
			//addFlag(castles.active[0].x-30, castles.active[0].y+20, 1);
			//for(i = 0; i < 5; i++) addUnit(flags.active[0].x+Math.random()*80-40, flags.active[0].y+Math.random()*80-40, 1);
			
			castles.active[1].setTeam(2);
			castles.active[1].hp = 100;
			for(i = 0; i < 5; i++) castles.active[1].makeUnit();
			//addFlag(castles.active[1].x-30, castles.active[1].y+20, 2);
			//for(i = 0; i < 5; i++) addUnit(flags.active[1].x+Math.random()*80-40, flags.active[1].y+Math.random()*80-40, 2);
			
			/*castles.active[2].setTeam(3);
			addFlag(castles.active[2].x-30, castles.active[2].y+20, 3);
			
			castles.active[3].setTeam(4);
			addFlag(castles.active[3].x-30, castles.active[3].y+20, 4);
			
			castles.active[4].setTeam(5);
			addFlag(castles.active[4].x-30, castles.active[4].y+20, 5);*/
			
			borders.reset();
			var lines:Vector.<LineSegment> = voronoi.voronoiDiagram();
			l = lines.length;
			for(i = 0; i < l; i++) {
				drawLine(borders, lines[i].p0.x, lines[i].p0.y, lines[i].p1.x, lines[i].p1.y, 0xf9e2bb, 4, 1.0);
			}
			borders.blendMode = "multiply";
			//borders.alpha = 0.5;
			
			/*lines = voronoi.delaunayLinesForSite(points[0]);
			l = lines.length;
			for(i = 0; i < l; i++) {
				drawLine(borders, lines[i].p0.x, lines[i].p0.y, lines[i].p1.x, lines[i].p1.y, 0xff0000, 4, 1.0);
			}*/
			
			//for(i = 0; i < 100; i++) addUnit(100+Math.random()*824, 100+Math.random()*568, 0);
			
			gameOver = false;
			
			addEventListener(Event.ENTER_FRAME, update);
			addEventListener(TouchEvent.TOUCH, on_touch);
		}
		
		public function initCastles(num:int):void {
			var l:int = num;
			for(var i:int = 0; i < l; i++) {
				var x:Number = Math.random()*824 + 100;
				var y:Number = Math.random()*568 + 100;
				var overflow:int = 0;
				while(overlap(x, y) && overflow < 1000) {
					x = Math.random()*824 + 100;
					y = Math.random()*568 + 100;
					overflow++;
				}
				if(overflow < 1000) addCastle(x, y, 0);
			}
		}
		
		public function overlap(x:Number, y:Number):Boolean {
			var l:int = castles.active.length;
			for(var i:int = 0; i < l; i++) {
				var castle:Castle = castles.active[i] as Castle;
				if(getDistance(castle.x, castle.y, x, y) < 180) return true;
			}
			return false;
		}
		
		public function deInit():void {
			var l:int = entities.length;
			for(var i:int = 0; i < l; i++) entities[i].deInit();
			
			removeEventListener(Event.ENTER_FRAME, update);
			removeEventListener(TouchEvent.TOUCH, on_touch);
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// TOUCH EVENTS
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		public var selected:Flag;
		public var currentCastle:Castle;
		public var hoverCastle:Castle;
		public var mouseX:Number;
		public var mouseY:Number;
		
		public function on_touch(e:TouchEvent):void {
			var touch:Touch = e.getTouch(touchQuad);
			if(touch) {
				mouseX = touch.globalX;
				mouseY = touch.globalY;
			} else currentCastle = null;
			touchHover(e);
			touchBegan(e);
			touchMoved(e);
			touchEnded(e);
		}
		
		public function touchHover(e:TouchEvent):void {
			var touch:Touch = e.getTouch(touchQuad, TouchPhase.HOVER);
			if(!touch) return;
			
			currentCastle = getClosestCastle(touch.globalX, touch.globalY);
			if(currentCastle.team != 1) currentCastle = null;
		}
		
		public function touchBegan(e:TouchEvent):void {
			var touch:Touch = e.getTouch(touchQuad, TouchPhase.BEGAN);
			if(!touch) return;
			
			currentCastle = getClosestCastle(touch.globalX, touch.globalY);
			if(currentCastle && currentCastle.team != 1) currentCastle = null; 
			hoverCastle = null;
		}
		
		public function touchMoved(e:TouchEvent):void {
			var touch:Touch = e.getTouch(touchQuad, TouchPhase.MOVED);
			if(!touch) return;
			
			if(currentCastle) hoverCastle = getClosestCastle(touch.globalX, touch.globalY);
		}
		
		public function touchEnded(e:TouchEvent):void {
			var touch:Touch = e.getTouch(touchQuad, TouchPhase.ENDED);
			if(!touch) return;
			
			if(currentCastle) {
				hoverCastle = getClosestCastle(touch.globalX, touch.globalY);
				currentCastle.moveUnits(hoverCastle);
			}
			currentCastle = null;
			hoverCastle = null;
		}
		
		public function getClosestTeamFlag(x:Number, y:Number, team:int):Flag {
			var closest:Flag;
			var min:Number = 60;
			
			var l:int = flags.active.length;
			for(var i:int = 0; i < l; i++) {
				var flag:Flag = flags.active[i] as Flag;
				if(flag.team != team) continue;
				var dist:Number = getDistance(flag.x, flag.y, x, y);
				if(dist < min) {
					min = dist;
					closest = flag;
				}
			}
			return closest;
		}
		
		public function getClosestCastle(x:Number, y:Number):Castle {
			var l:int = castles.activeIndex;
			for(var i:int = 0; i < l; i++) {
				if(castles.active[i].contains(x, y)) {
					return castles.active[i] as Castle;
				}
			}
			return null;
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// UPDATES
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		public function update(e:EnterFrameEvent):void {
			var dt:Number = e.passedTime;
			updateStats();
			var l:int = entities.length;
			for(var i:int = 0; i < l; i++) entities[i].update(dt);
			
			paths.reset();
			if(selected) {
				l = selected.waypointIndex-1;
				for(i = 0; i < l; i++) {
					drawLine(paths, selected.waypoints[i].x, selected.waypoints[i].y, selected.waypoints[i+1].x, selected.waypoints[i+1].y, 0x0000ff, 2, 0.5);
				}
				if(l > 0) {
					drawLine(paths, selected.waypoints[i].x, selected.waypoints[i].y, mouseX, mouseY, 0x0000ff, 2, 0.5);
					drawLine(paths, selected.waypoints[0].x, selected.waypoints[0].y, selected.x, selected.y, 0x0000ff, 2, 0.5);
				}
			}
			
			l = flags.active.length;
			for(i = 0; i < l; i++) {
				var flag:Flag = flags.active[i] as Flag;
				var k:int = flag.waypointIndex-1;
				for(var j:int = 0; j < k; j++) {
					drawLine(paths, flag.waypoints[j].x, flag.waypoints[j].y, flag.waypoints[j+1].x, flag.waypoints[j+1].y, 0x0000ff, 2, 0.25);
				}
				if(flag.waypointIndex > 0) drawLine(paths, flag.x, flag.y, flag.waypoints[0].x, flag.waypoints[0].y, 0x0000ff, 2, 0.25);
			}
			
			l = castles.activeIndex;
			for(i = 0; i < l; i++) {
				castles.active[i].regionHighlight.visible = false;
			}
			if(currentCastle) currentCastle.regionHighlight.visible = true;
			
			ui_arrow.visible = false;
			if(hoverCastle && hoverCastle != currentCastle && currentCastle.adjacent.indexOf(hoverCastle) != -1) {
				hoverCastle.regionHighlight.visible = true;
				ui_arrow.x = currentCastle.x;
				ui_arrow.y = currentCastle.y;
				ui_arrow.rotation = 0;
				ui_arrow.scaleX = ui_arrow.scaleY = 1.0;
				ui_arrow.width = getDistance(currentCastle.x, currentCastle.y, hoverCastle.x, hoverCastle.y);
				ui_arrow.rotation = getAngle(currentCastle.x, currentCastle.y, hoverCastle.x, hoverCastle.y);
				ui_arrow.visible = true;
			}
			
			unitLayer.sortChildren(compare);
			GS.update(dt);
			if(!gameOver) checkGameOver();
		}
		
		public function compare(a:DisplayObject, b:DisplayObject):Number {
			return a.y - b.y;
		}
		
		public function updateStats():void {
			var l:int = Globals.teamCaps.length;
			for(var i:int = 0; i < l; i++) {
				Globals.teamCaps[i] = 0;
				Globals.teamPops[i] = 0;
			}
			
			l = castles.activeIndex;
			for(i = 0; i < l; i++) {
				Globals.teamCaps[castles.active[i].team] += 10;
			}
			
			l = units.activeIndex;
			for(i = 0; i < l; i++) {
				Globals.teamPops[units.active[i].team]++;
			}
		}
		
		public function checkGameOver():void {
			if(Globals.teamCaps[1] == 0) {
				gameOver = true;
				defeat.visible = true;
			}
			
			if(Globals.teamCaps[2] == 0) {
				gameOver = true;
				victory.visible = true;
			}
		}
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		// GAME LOGIC
		//////////////////////////////////////////////////////////////////////////////////////////////////////////
		
		public function addUnit(x:Number, y:Number, team:int, castle:Castle):Unit {
			var unit:Unit = units.getReserve() as Unit;
			if(!unit) unit = new Unit();
			unit.initUnit(this, x, y, team, castle);
			units.addEntity(unit);
			return unit;
		}
		
		public function addFlag(x:Number, y:Number, team:int):Flag {
			var flag:Flag = flags.getReserve() as Flag;
			if(!flag) flag = new Flag();
			flag.initFlag(this, x, y, team);
			flags.addEntity(flag);
			return flag;
		}
		
		public function addCastle(x:Number, y:Number, team:int):Castle {
			var castle:Castle = castles.getReserve() as Castle;
			if(!castle) castle = new Castle();
			castle.initCastle(this, x, y, team);
			castles.addEntity(castle);
			return castle;
		}
		
		public function addArrow(x:Number, y:Number, target:Unit):Arrow {
			var arrow:Arrow = arrows.getReserve() as Arrow;
			if(!arrow) arrow = new Arrow();
			arrow.initArrow(this, x, y, target);
			arrows.addEntity(arrow);
			return arrow;
		}
		
		public var lineQuad:Quad;
		public function drawLine(batch:QuadBatch, x1:Number, y1:Number, x2:Number, y2:Number, color:uint, thickness:Number=2, alpha:Number=1):void {
			if(!lineQuad) lineQuad = new Quad(10, 10, 0xffffff);
			var quad:Quad = lineQuad;
			quad.color = color;
			quad.alpha = alpha;
			quad.rotation = 0;
			thickness *= 0.5;
			
			var dx:Number = x2 - x1;
			var dy:Number = y2 - y1;
			var angle:Number = Math.atan2(dy, dx);
			var dist:Number = Math.sqrt(dx*dx +dy*dy);
			quad.x = x1;
			quad.y = y1;
			quad.setVertexPosition(0, 0, -thickness);
			quad.setVertexPosition(1, dist, -thickness);
			quad.setVertexPosition(2, 0, thickness);
			quad.setVertexPosition(3, dist, thickness);
			quad.rotation = angle;
			batch.addQuad(quad);
		}
		
		public function getDistance(x1:Number, y1:Number, x2:Number, y2:Number):Number {
			var dx:Number = x2 - x1;
			var dy:Number = y2 - y1;
			return Math.sqrt(dx*dx + dy*dy);
		}
		
		public function getAngle(x1:Number, y1:Number, x2:Number, y2:Number):Number {
			var dx:Number = x2 - x1;
			var dy:Number = y2 - y1;
			return Math.atan2(dy, dx);
		}
		
		public function fastDist(x1:Number, y1:Number, x2:Number, y2:Number):Number {
			var dx:Number = x2 - x1;
			var dy:Number = y2 - y1;
			return dx*dx+dy*dy;
		}
		
	}
}